package com.nd.idaas.demo.redirect.demo1;

import com.nd.idaas.demo.config.IdaasConfig;
import com.nd.idaas.sdk.saml2.core.GenerateArtifactResolveSamlProtocols;
import com.nd.idaas.sdk.saml2.core.GenerateLogoutRequestSamlProtocols;
import com.nd.idaas.sdk.saml2.core.GenerateRequestSamlProtocols;
import com.nd.idaas.sdk.saml2.sp.SpProvider;
import com.nd.idaas.sdk.saml2.sp.SpUtils;
import com.nd.idaas.sdk.saml2.utils.IdaasUtils;
import com.nd.idaas.sdk.saml2.utils.RedirectUtils;
import com.nd.idaas.sdk.saml2.utils.SAMLResolver;
import com.nd.idaas.sdk.saml2.vo.SAMLResponse;
import org.opensaml.saml.saml2.core.ArtifactResponse;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * DEMO1：前后端分离，SP与IDP分离.
 * <p>Description: </p>
 * <p>Create Time: 2017/11/16 0016 10:47 </p>
 * @author 910204
 */
@RestController(value = "RedirectDemo1Controller2")
@RequestMapping("/redirect/demo1")
public class SpController {

    //IDAAS 校验成功后跳转地址
    private final String LOGIN_SUCCESS_URL = "http://192.168.248.122:8880/redirect/demo1/sp/login_success";
    private final String LOGOUT_SUCCESS_URL = "http://192.168.248.122:8880/redirect/demo1/sp/logout_success";
    {
        //注册SP服务
        SpProvider.init(IdaasConfig.PUBLICKEY,IdaasConfig.PRIVATEKEY, IdaasConfig.IDAAS_HOST, IdaasConfig.IDAAS_WEB_HOST,IdaasConfig.IDAASWEBCLIENTID,
                IdaasConfig.IDAASANDROIDCLIENTID, IdaasConfig.IDAASIOSCLIENTID);
    }

    /**
     * 步骤一：sp发起idaas认证请求，并跳转到idaas
     */
    @RequestMapping(value = "/sp/step_1", method = RequestMethod.GET)
    public Object spStep1() {
        // 构建请求协议
        GenerateRequestSamlProtocols protocols =
                new GenerateRequestSamlProtocols(LOGIN_SUCCESS_URL, IdaasConfig.IDAASWEBCLIENTID, IdaasConfig.PUBLICKEY, IdaasConfig.PRIVATEKEY);

        // 生成请求的完整URL
        String idpRequestUriFull = RedirectUtils.getUrlSignedRequestEncodedUrl(protocols);
        StringBuilder fullUriSb = new StringBuilder(idpRequestUriFull);
        fullUriSb.append("&product=").append(IdaasConfig.IDAASPRODUCTCODE).append("&to_client_id=").append(IdaasConfig.IDAASWEBCLIENTID);

        return fullUriSb.toString();
    }


    /**
     * 步骤四：idaas接收并处理response后，生成response，并通过模拟post表单模式请求sp接口
     */
    @RequestMapping(value = "/sp/login_success", method = RequestMethod.GET)
    public void spStep2(@RequestParam(value = "SAMLArtifact") String samlArtifact, HttpServletResponse response, RedirectAttributes attr) throws Exception {
        String ProtocolStatus = "urn:oasis:names:tc:SAML:2.0:status:Success";
        try {
            // 解析ArtifactResponse
            GenerateArtifactResolveSamlProtocols protocols =
                    new GenerateArtifactResolveSamlProtocols(samlArtifact, IdaasConfig.IDAASWEBCLIENTID, SpProvider.getIdaasHost());
            ArtifactResponse responseVo = IdaasUtils.soapGetArtifactResponse(protocols, response);
            if (SpUtils.isStatusSuccessful(responseVo)) {
                // 解析ArtifactResponse
                SAMLResponse samlResponse =
                        SAMLResolver.resolveArtifactResponse(responseVo, IdaasConfig.PUBLICKEY, IdaasConfig.PRIVATEKEY);
                // 获取idp的用户ID
                String idpUserId = samlResponse.getSubjectForNameID();

                /*
                 //向idp兑换token
                GenerateResponseSamlProtocols responseProtocol =
                        new GenerateResponseSamlProtocols(Test.PUBLICKEY, Test.PRIVATEKEY, "http://cms-admin.101.com/" + UcServiceUri.SAML2_EXCHANGE_TOKENS);
                responseProtocol.setStatus(ProtocolStatus);
                responseProtocol.setIdaasTenantId(Test.IDAASWEBCLIENTID);
                responseProtocol.setSubjectForNameID(idpUserId);
                **/

                //转转到idp
                response.sendRedirect("http://www.baidu.com");
            }else{
                response.sendRedirect("http://www.baidu.com");
            }
        } catch (RuntimeException e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*********************************登出操作**********************************/

    /**
     * 构建登出请求的SAML协议，并由前端跳转到idaas
     */
    @RequestMapping(value = "/sp/logout/step_1", method = RequestMethod.GET)
    public Object spLogoutStep1(HttpServletRequest request) {
        String userid = request.getParameter("userid");
        //构建协议请求
        GenerateLogoutRequestSamlProtocols protocols =
                new GenerateLogoutRequestSamlProtocols(LOGOUT_SUCCESS_URL, IdaasConfig.IDAASWEBCLIENTID, String.valueOf(userid), IdaasConfig.PUBLICKEY, IdaasConfig.PRIVATEKEY);

        //生成请求的完整URL
        String idpLogoutRequestUriFull = RedirectUtils.getUrlSignedRequestEncodedUrl(protocols);

        // 前端接收到该请求后执行跳转
        return idpLogoutRequestUriFull;
    }

    /**
     * idaas处理登出请求成功后，回调在step_1中设置的spResponseUri
     */
    @RequestMapping(value = "/sp/logout_success", method = RequestMethod.POST)
    public void resolveLogoutResponse(HttpServletRequest request, HttpServletResponse httpServletResponse) throws Exception{
//        SAMLResponse samlResponse = SpUtils.resolveResponse(request);
        // TODO 向idp执行登出操作
        httpServletResponse.sendRedirect("http://www.baidu.com/logout");
//        return samlResponse.getSubjectForNameID();
    }
}
